{% extends "pretixcontrol/organizers/base.html" %}
{% load i18n %}
{% load bootstrap3 %}
{% block inner %}
    <h1>{% trans "Revoke device access:" %} {{ device.name }}</h1>
    <form action="" method="post" class="form-horizontal">
        {% csrf_token %}
        <p>
            <strong>{% blocktrans %}Are you sure you want remove access for this device?{% endblocktrans %}</strong>
            {% trans "All data of this device will stay available, but you can't use the device any more." %}
        </p>
        <div class="alert alert-warning">
            <ul>
                <li>{% trans "All data uploaded by this device will stay available online." %}</li>
                <li>{% trans "If data (e.g. POS transactions or check-ins) has been created on this device and has not been uploaded, you will no longer be able to upload it." %}</li>
                <li>{% trans "If the device software supports it, personal data such as orders will be deleted from the device on the next synchronization attempt. Non-personal data such as event metadata and POS transactions will persist until you uninstall or reset the software manually." %}</li>
            </ul>
        </div>
        <div class="form-group submit-group">
            <a href="{% url "control:organizer.devices" organizer=request.organizer.slug %}" class="btn btn-default btn-cancel">
                {% trans "Cancel" %}
            </a>
            <button type="submit" class="btn btn-danger btn-save">
                {% trans "Revoke" %}
            </button>
        </div>
    </form>
{% endblock %}
