# Generated by Django 3.0.6 on 2020-06-20 16:33

import django_countries.fields
from django.db import migrations, models

import pretix.helpers.countries


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0153_auto_20200528_1953'),
    ]

    operations = [
        migrations.AddField(
            model_name='subeventitem',
            name='disabled',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='subeventitemvariation',
            name='disabled',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='invoiceaddress',
            name='country',
            field=django_countries.fields.CountryField(countries=pretix.helpers.countries.CachedCountries, max_length=2),
        ),
    ]
