# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-06-04 11:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0001_initial'),
        ('pretixapi', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='oauthaccesstoken',
            name='organizers',
            field=models.ManyToManyField(to='pretixbase.Organizer'),
        ),
        migrations.AddField(
            model_name='oauthgrant',
            name='organizers',
            field=models.ManyToManyField(to='pretixbase.Organizer'),
        ),
    ]
