{% extends "error.html" %}
{% load i18n %}
{% load static %}
{% block title %}{% trans "Verification failed" %}{% endblock %}
{% block content %}
    <i class="fa fa-frown-o big-icon fa-fw"></i>
    <div class="error-details">
        <h1>{% trans "Verification failed" %}</h1>
        <p>{% blocktrans trimmed %}
            We could not verify that this request really was sent from you. For security reasons, we therefore cannot
            process it.
        {% endblocktrans %}</p>
        {% if no_referer %}
            <p>{{ no_referer1 }}</p>
            <p>{{ no_referer2 }}</p>
        {% elif no_cookie %}
            <p>{{ no_cookie1 }}</p>
            <p>{{ no_cookie2 }}</p>
        {% else %}
            <p>{% blocktrans trimmed %}
                Please go back to the last page, refresh this page and then try again. If the problem persists, please
                get in touch with us.
            {% endblocktrans %}</p>
        {% endif %}
        <p class="links">
            <a id='goback' href='#'>{% trans "Take a step back" %}</a>
        </p>
        <img src="{% static "pretixbase/img/pretix-logo.svg" %}" class="logo"/>
    </div>
{% endblock %}
