# Generated by Django 2.2.11 on 2020-03-20 14:05

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0146_giftcardtransaction_text'),
        ('pretixmultidomain', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='knowndomain',
            name='event',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='domains', to='pretixbase.Event'),
        ),
    ]
