# Generated by Django 2.1.5 on 2019-02-01 14:24

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('badges', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='badgelayout',
            options={'ordering': ('name',)},
        ),
        migrations.AlterField(
            model_name='badgeitem',
            name='layout',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='item_assignments', to='badges.BadgeLayout'),
        ),
        migrations.AlterField(
            model_name='badgelayout',
            name='default',
            field=models.BooleanField(default=False, verbose_name='Default'),
        ),
        migrations.AlterField(
            model_name='badgelayout',
            name='layout',
            field=models.TextField(default='[{"type":"textarea","left":"13.09","bottom":"49.73","fontsize":"23.6","color":[0,0,0,1],"fontfamily":"Open Sans","bold":true,"italic":false,"width":"121.83","content":"attendee_name","text":"Max Mustermann","align":"center"}]'),
        ),
    ]
