# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-08-22 10:44
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0032_item_allow_cancel'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item',
            name='allow_cancel',
            field=models.BooleanField(default=True, help_text='Whether this product might or not be cancelled.', verbose_name='Allow product to be cancelled'),
        ),
    ]
