# Generated by Django 3.0.11 on 2020-12-22 10:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0170_remove_hidden_urls'),
        ('pretixbase', '0162b_auto_20201218_1810'),
    ]

    operations = [
    ]
