# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-05-04 07:06
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0058_auto_20170429_1020'),
    ]

    operations = [
        migrations.AddField(
            model_name='checkin',
            name='nonce',
            field=models.CharField(blank=True, max_length=190, null=True),
        ),
    ]
