{% extends "pretixcontrol/event/base.html" %}
{% load i18n %}
{% block title %}
    {% trans "Cancel refund" %}
{% endblock %}
{% block content %}
    <h1>
        {% trans "Cancel refund" %}
    </h1>
    <p>{% blocktrans trimmed %}
        Do you really want to cancel this refund? You cannot revert this action.
    {% endblocktrans %}</p>
    <div class="alert alert-info">
        {% blocktrans trimmed %}
            If the money is already on the way back, this will not stop the money, it will just mark this transfer as
            aborted in pretix. This will also not reactivate the order, it will just allow you to choose a new refund
            method.
        {% endblocktrans %}
    </div>

    <form method="post" href="">
        {% csrf_token %}
        <div class="row checkout-button-row">
            <div class="col-md-4">
                <a class="btn btn-block btn-default btn-lg"
                        href="{% url "control:event.order" event=request.event.slug organizer=request.event.organizer.slug code=order.code %}">
                    {% trans "No, take me back" %}
                </a>
            </div>
            <div class="col-md-4 col-md-offset-4">
                <button class="btn btn-block btn-danger btn-lg" type="submit">
                    {% trans "Yes, cancel refund" %}
                </button>
            </div>
            <div class="clearfix"></div>
        </div>
    </form>
{% endblock %}
