{% load i18n %}

<p>
    {% blocktrans %}
        Your gift card will be used to pay for this order. If the credit on the gift card is lower than the order total, you will be able to pay the
        difference with a different payment method. If the credit is higher than the order total, you will be able to re-use the gift card in the future.
    {% endblocktrans %}
</p>
