# Generated by Django 2.2 on 2019-09-18 17:42

from django.db import migrations, models

import pretix.base.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0135_auto_20191007_0803'),
    ]

    operations = [
        migrations.AddField(
            model_name='checkin',
            name='auto_checked_in',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='checkinlist',
            name='auto_checkin_sales_channels',
            field=pretix.base.models.fields.MultiStringField(default=[]),
        )
    ]
