# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-02-22 20:02
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0008_invoiceaddress'),
    ]

    operations = [
        migrations.AddField(
            model_name='event',
            name='live',
            field=models.BooleanField(default=False, verbose_name='Shop is live'),
        ),
        migrations.AlterField(
            model_name='invoiceaddress',
            name='order',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_address', to='pretixbase.Order'),
        ),
    ]
