# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-02-20 10:31
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0080_question_ask_during_checkin'),
    ]

    operations = [
        migrations.AddField(
            model_name='checkinlist',
            name='include_pending',
            field=models.BooleanField(default=False, verbose_name='Include pending orders'),
        ),
        migrations.AlterField(
            model_name='event',
            name='presale_end',
            field=models.DateTimeField(blank=True, help_text='Optional. No products will be sold after this date. If you do not set this value, the presale will end after the end date of your event.', null=True, verbose_name='End of presale'),
        ),
        migrations.AlterField(
            model_name='logentry',
            name='event',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='pretixbase.Event'),
        ),
        migrations.AlterField(
            model_name='question',
            name='ask_during_checkin',
            field=models.BooleanField(default=False, help_text='This will only work if you handle your check-in with pretixdroid 1.8 or newer or pretixdesk 0.2 or newer.', verbose_name='Ask during check-in instead of in the ticket buying process'),
        ),
        migrations.AlterField(
            model_name='subevent',
            name='presale_end',
            field=models.DateTimeField(blank=True, help_text='Optional. No products will be sold after this date. If you do not set this value, the presale will end after the end date of your event.', null=True, verbose_name='End of presale'),
        ),
        migrations.AlterField(
            model_name='user',
            name='require_2fa',
            field=models.BooleanField(default=False, verbose_name='Two-factor authentification is required to log in'),
        ),
    ]
