# Generated by Django 2.1 on 2019-01-12 15:12

import django.db.models.deletion
import jsonfallback.fields
from django.db import migrations, models

import pretix.base.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0104_auto_20181114_1526'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoiceaddress',
            name='beneficiary',
            field=models.TextField(blank=True, verbose_name='Beneficiary'),
        ),
        migrations.AddField(
            model_name='invoice',
            name='invoice_to_beneficiary',
            field=models.TextField(blank=True, null=True, verbose_name='Beneficiary'),
        ),
    ]
