# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-03-20 12:19
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixhelpers', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='thumbnail',
            name='thumb',
            field=models.FileField(max_length=255, upload_to='pub/thumbs/'),
        ),
    ]
