# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-05-11 12:24
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('pretixbase', '0091_auto_20180513_1641'),
    ]

    operations = [
        migrations.AddField(
            model_name='item',
            name='original_price',
            field=models.DecimalField(blank=True, decimal_places=2,
                                      help_text='If set, this will be displayed next to the current price to show '
                                                'that the current price is a discounted one. This is just a cosmetic '
                                                'setting and will not actually impact pricing.',
                                      max_digits=7, null=True, verbose_name='Original price'),
        ),
    ]
