html.rtl {
  .info-row {
    & > .fa {
      float: right;
    }
    p {
      margin-right: 40px;
      margin-left: 0;
    }
  }
  .cart-row > div {
    float: right;
  }
  .checkout-flow .checkout-step:last-child .checkout-step-bar-right, .checkout-flow .checkout-step:first-child .checkout-step-bar-left {
    background: $gray-lighter;
  }
  .checkout-flow .checkout-step.step-current .checkout-step-bar-left {
    background: $gray-lighter;
  }
  .checkout-flow .checkout-step.step-current .checkout-step-bar-right,
  .checkout-flow .checkout-step.step-done .checkout-step-bar-right, .checkout-flow .checkout-step.step-done .checkout-step-bar-left {
    background: $brand-primary;
  }
  .checkout-flow .checkout-step:last-child .checkout-step-bar-left, .checkout-flow .checkout-step:first-child .checkout-step-bar-right {
    background: transparent;
  }
  .cart-modify {
    margin-left: 0;
    margin-right: 10px;
  }
  .cart-row dl dd {
    padding-right: 20px;
    padding-left: 0;
  }
  .thank-you .fa {
    float: right;
    margin-left: 30px;
    margin-right: 0;
  }
  .thank-you p {
    margin-right: 158px;
    margin-left: 0;
  }
  .fa-arrow-right::before {
    content: $fa-var-arrow-left;
  }
  .fa-arrow-left::before {
    content: $fa-var-arrow-right;
  }
  .page-header h1, h2.content-header {
    direction: rtl;
    * {
      display: inline-block;
    }
  }
}

