# Generated by Django 2.2.1 on 2019-05-15 13:23

from django.db import migrations, models

import pretix.base.models.orders


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0121_order_email_known_to_work'),
    ]

    operations = [
        migrations.AddField(
            model_name='orderposition',
            name='web_secret',
            field=models.CharField(db_index=True, default=pretix.base.models.orders.generate_secret, max_length=32),
        ),
    ]
