# Generated by Django 2.1 on 2018-10-23 23:00

import django_countries.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0099_auto_20180912_1035'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoice',
            name='invoice_from_city',
            field=models.CharField(max_length=190, null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='invoice_from_country',
            field=django_countries.fields.CountryField(max_length=2, null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='invoice_from_name',
            field=models.CharField(max_length=190, null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='invoice_from_tax_id',
            field=models.CharField(max_length=190, null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='invoice_from_vat_id',
            field=models.CharField(max_length=190, null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='invoice_from_zipcode',
            field=models.CharField(max_length=190, null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='invoice_to_city',
            field=models.TextField(null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='invoice_to_company',
            field=models.TextField(null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='invoice_to_country',
            field=django_countries.fields.CountryField(max_length=2, null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='invoice_to_name',
            field=models.TextField(null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='invoice_to_street',
            field=models.TextField(null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='invoice_to_vat_id',
            field=models.TextField(null=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='invoice_to_zipcode',
            field=models.CharField(max_length=190, null=True),
        ),
    ]
