{% load static %}
{% load compress %}
{% load i18n %}

{% compress js %}
    <script type="text/javascript" src="{% static "pretixplugins/stripe/pretix-stripe.js" %}"></script>
{% endcompress %}
{% compress css %}
    <link type="text/css" rel="stylesheet" href="{% static "pretixplugins/stripe/pretix-stripe.css" %}">
{% endcompress %}
{% if testmode %}
    <script type="text/plain" id="stripe_pubkey">{{ settings.publishable_test_key }}</script>
{% else %}
    <script type="text/plain" id="stripe_pubkey">{{ settings.publishable_key }}</script>
{% endif %}
{% if settings.connect_user_id %}
    <script type="text/plain" id="stripe_connectedAccountId">{{ settings.connect_user_id }}</script>
{% endif %}
    <script type="text/plain" id="stripe_merchantcountry">{{ settings.merchant_country|default:"" }}</script>
