{% load compress %}
{% load i18n %}
{% load static %}
<!DOCTYPE html>
<html>
<head>
    <title>{{ settings.PRETIX_INSTANCE_NAME }}</title>
    {% compress css %}
        <link rel="stylesheet" type="text/x-scss" href="{% static "pretixpresale/scss/waiting.scss" %}"/>
    {% endcompress %}
    {% compress js %}
        <script type="text/javascript" src="{% static "jquery/js/jquery-2.1.1.min.js" %}"></script>
        <script type="text/javascript" src="{% static "pretixbase/js/ajaxpending.js" %}"></script>
    {% endcompress %}
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta http-equiv="refresh" content="10">
</head>
<body>
    <div class="container">
        <i class="fa fa-cog big-rotating-icon"></i>

        <h1>{% trans "We are processing your request …" %}</h1>

        <p>
            {% trans "If this takes longer than a few minutes, please contact us." %}
        </p>
    </div>
</body>
</html>
