# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-05-09 09:17
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('pretixbase', '0089_auto_20180315_1322'),
    ]

    operations = [
        migrations.AddField(
            model_name='item',
            name='internal_name',
            field=models.CharField(blank=True,
                                   help_text='If you set this, this will be used instead of the public name in the '
                                             'backend.',
                                   max_length=255, null=True, verbose_name='Internal name'),
        ),
        migrations.AddField(
            model_name='itemcategory',
            name='internal_name',
            field=models.CharField(blank=True,
                                   help_text='If you set this, this will be used instead of the public name in the backend.',
                                   max_length=255, null=True, verbose_name='Internal name'),
        ),
    ]
