# Generated by Django 2.1.1 on 2018-11-21 12:24

import django.db.models.deletion
import jsonfallback.fields
from django.db import migrations, models

import pretix.base.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0102_auto_20181017_0024'),
    ]

    operations = [
        migrations.AddField(
            model_name='item',
            name='sales_channels',
            field=pretix.base.models.fields.MultiStringField(default=['web'], verbose_name='Sales channels'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='order',
            name='sales_channel',
            field=models.CharField(default='web', max_length=190),
            preserve_default=False,
        ),
    ]
