# Generated by Django 3.0.6 on 2020-05-28 19:53

import django_countries.fields
from django.db import migrations, models

import pretix.helpers.countries


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0152_auto_20200511_1504'),
    ]

    operations = [
        migrations.AddField(
            model_name='seat',
            name='x',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='seat',
            name='y',
            field=models.FloatField(null=True),
        ),
        migrations.AlterField(
            model_name='seat',
            name='seat_guid',
            field=models.CharField(db_index=True, max_length=190),
        ),
    ]
