# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-08-31 21:04
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0033_auto_20160822_1044'),
        ('pretixbase', '0034_auto_20160830_1952'),
    ]

    operations = [
    ]
