{% extends "pretixcontrol/auth/base.html" %}
{% load bootstrap3 %}
{% load static %}
{% load i18n %}
{% block content %}
	<form class="form-signin" action="" method="post">
        <h3>{% trans "Set new password" %}</h3>
        {% csrf_token %}
        {% bootstrap_form_errors form type='all' layout='inline' %}
        {% bootstrap_field form.password %}
        {% bootstrap_field form.password_repeat %}
		<div class="form-group buttons">

            <button type="submit" class="btn btn-primary">
                {% trans "Save" %}
            </button>
		</div>
	</form>
{% endblock %}
