# Generated by Django 2.2 on 2019-04-23 08:39

import django.db.models.deletion
import jsonfallback.fields
from django.db import migrations, models

import pretix.base.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0117_auto_20190418_1149'),
    ]

    operations = [
        migrations.AddField(
            model_name='subevent',
            name='is_public',
            field=models.BooleanField(default=True, help_text='If selected, this event will show up publicly on the list of dates for your event.', verbose_name='Show in lists'),
        ),
    ]
