@import "_variables.scss";
@import "../../bootstrap/scss/_bootstrap.scss";
@import "_theme.scss";
@import "../../fontawesome/scss/font-awesome.scss";


body {
    background: #fbf7fc;
    padding: 50px 0;
}

.big-icon {
    font-size: 200px;
    color: $navbar-inverse-bg;
    display: block;
    float: left;
}

.error-details {
    margin-left: 260px;
}

.container {
    width: 90%;
    max-width: 800px;
}

.logo {
    height: 35px;
    width: auto;
}

@media(max-width: 600px) {
    body {
        text-align: center;
    }
    .big-icon {
        margin: auto;
        float: none;
    }
    .error-details {
        margin-left: 0;
    }
}
