# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-07-28 17:25
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0023_auto_20160601_1039'),
    ]

    operations = [
        migrations.AddField(
            model_name='item',
            name='hide_without_voucher',
            field=models.BooleanField(default=False, help_text='This product will be hidden from the event page until the user enters a voucher code that is specifically tied to this product (and not via a quota).', verbose_name='This product will only be shown if a voucher is redeemed.'),
        ),
        migrations.AddField(
            model_name='item',
            name='require_voucher',
            field=models.BooleanField(default=False, help_text='To buy this product, the user needs a voucher that applies to this product either directly or via a quota.', verbose_name='This product can only be bought using a voucher.'),
        ),
    ]
