{% load i18n %}

<p>{% blocktrans trimmed %}
    After you clicked continue, we will redirect you to PayPal to fill in your payment
    details. You will then be redirected back here to review and confirm your order.
{% endblocktrans %}</p>
