a.btn, button.btn {
  max-width: 100%;
  white-space: normal;
}

.panel-title .radio {
  margin-left: 20px;
}

.form-control + .form-control-feedback {
  /* Fix for https://github.com/FortAwesome/Font-Awesome/issues/4313 */
  position: absolute;
  top: 0;
  right: 0;
  z-index: 2; // Ensure icon is above input groups
  display: block;
  width: $input-height-base;
  height: $input-height-base;
  line-height: $input-height-base;
  text-align: center;
  pointer-events: none;
}

.js-copy-answers {
  margin-right: 10px;
}

.form-control.is-focused {
  @extend .form-control:focus;
}

.form-control.has-error {
  border-color: $state-danger-text;
  @include box-shadow(inset 0 1px 1px rgba(0, 0, 0, .075)); // Redeclare so transitions work
  &:focus, &.is-focused {
    border-color: darken($state-danger-text, 10%);
    $shadow: inset 0 1px 1px rgba(0, 0, 0, .075), 0 0 6px lighten($state-danger-text, 20%);
    @include box-shadow($shadow);
  }
}

.required-legend span {
  color: $brand-primary;
  font-weight: bold;
}
.form-group.required .control-label:after {
  content: '*';
  color: $brand-primary;
}
.form-control-text {
  padding-top: 7px;
}
.addon-list {
  padding: 0;
  list-style-type: none;
}
.accordion-radio {
  display: block;
  margin: 0;
}
.panel-default>.accordion-radio>.panel-heading {
  color: #333;
  background-color: #f5f5f5;
  padding: 12px 15px;

  input[type=radio] {
    margin-top: 0;
    margin-right: 5px;
    position: relative;
    top: 2px;
  }
}
.panel-default>.accordion-radio+.panel-collapse>.panel-body {
  border-top: 1px solid #ddd;
}

.splitdatetimerow {
  display: flex;
  flex-direction: row;
  flex-wrap: wrap;

  .help-block {
    width: 100%;
  }
}
.splitdatetimepart {
  width: 50%;
  display: inline-block;

  &.datepickerfield {
    border-bottom-right-radius: 0;
    border-top-right-radius: 0;
  }
  &.timepickerfield {
    border-bottom-left-radius: 0;
    border-top-left-radius: 0;
    border-left: 0;
  }
}

@media(max-width: $screen-xs-max) {
  .nameparts-form-group {
    display: block;
    input:not(:first-child), select:not(:first-child) {
      border-top-right-radius: 0;
      border-top-left-radius: 0;
    }
    input:not(:last-child), select:not(:last-child) {
      border-bottom-right-radius: 0;
      border-bottom-left-radius: 0;
    }
  }
}
@media(min-width: $screen-sm-min) {
  .nameparts-form-group {
    display: flex;
    flex-direction: row;
    input, select {
      width: auto;
      flex-basis: 0;
      flex-grow: 1;
    }
    input:not(:first-child), select:not(:first-child) {
      border-bottom-left-radius: 0;
      border-top-left-radius: 0;
    }
    input:not(:last-child), select:not(:last-child)  {
      border-bottom-right-radius: 0;
      border-top-right-radius: 0;
    }
    [data-size="1"] {
      flex-grow: 1;
      flex-shrink: 4;
    }
    [data-size="2"] {
      flex-grow: 2;
      flex-shrink: 3;
    }
    [data-size="3"] {
      flex-grow: 3;
      flex-shrink: 2;
    }
    [data-size="4"] {
      flex-grow: 4;
      flex-shrink: 1;
    }
  }
}
@media (max-width: $screen-sm-max) {
  .form-horizontal .control-label {
    text-align: left;
  }
}
