{% extends "pretixcontrol/base.html" %}
{% load i18n %}
{% load bootstrap3 %}

{% block title %}{% trans "Global settings" %}{% endblock %}
{% block content %}
    <h1>{% trans "Global settings" %}</h1>
    {% block inner %}
    {% endblock %}
{% endblock %}
