@font-face {
    font-family: 'Open Sans';
    src: url(static('fonts/opensans_bold_macroman/OpenSans-Bold-webfont.eot'));
    src: url(static('fonts/opensans_bold_macroman/OpenSans-Bold-webfont.eot?#iefix')) format('embedded-opentype'),
         url(static('fonts/opensans_bold_macroman/OpenSans-Bold-webfont.woff')) format('woff'),
         url(static('fonts/opensans_bold_macroman/OpenSans-Bold-webfont.ttf')) format('truetype'),
         url(static('fonts/opensans_bold_macroman/OpenSans-Bold-webfont.svg#open_sansbold')) format('svg');
    font-weight: 700;
    font-style: normal;

}

@font-face {
    font-family: 'Open Sans';
    src: url(static('fonts/opensans_bolditalic_macroman/OpenSans-BoldItalic-webfont.eot'));
    src: url(static('fonts/opensans_bolditalic_macroman/OpenSans-BoldItalic-webfont.eot?#iefix')) format('embedded-opentype'),
         url(static('fonts/opensans_bolditalic_macroman/OpenSans-BoldItalic-webfont.woff')) format('woff'),
         url(static('fonts/opensans_bolditalic_macroman/OpenSans-BoldItalic-webfont.ttf')) format('truetype'),
         url(static('fonts/opensans_bolditalic_macroman/OpenSans-BoldItalic-webfont.svg#open_sansbold_italic')) format('svg');
    font-weight: 700;
    font-style: italic;

}

@font-face {
    font-family: 'Open Sans';
    src: url(static('fonts/opensans_italic_macroman/OpenSans-Italic-webfont.eot'));
    src: url(static('fonts/opensans_italic_macroman/OpenSans-Italic-webfont.eot?#iefix')) format('embedded-opentype'),
         url(static('fonts/opensans_italic_macroman/OpenSans-Italic-webfont.woff')) format('woff'),
         url(static('fonts/opensans_italic_macroman/OpenSans-Italic-webfont.ttf')) format('truetype'),
         url(static('fonts/opensans_italic_macroman/OpenSans-Italic-webfont.svg#open_sansitalic')) format('svg');
    font-weight: normal;
    font-style: italic;

}

@font-face {
    font-family: 'Open Sans';
    src: url(static('fonts/opensans_regular_macroman/OpenSans-Regular-webfont.eot'));
    src: url(static('fonts/opensans_regular_macroman/OpenSans-Regular-webfont.eot?#iefix')) format('embedded-opentype'),
         url(static('fonts/opensans_regular_macroman/OpenSans-Regular-webfont.woff')) format('woff'),
         url(static('fonts/opensans_regular_macroman/OpenSans-Regular-webfont.ttf')) format('truetype'),
         url(static('fonts/opensans_regular_macroman/OpenSans-Regular-webfont.svg#open_sansregular')) format('svg');
    font-weight: 400;
    font-style: normal;

}
