# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-05-13 16:41
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0090_auto_20180509_0917'),
    ]

    operations = [
        migrations.AddField(
            model_name='order',
            name='last_modified',
            field=models.DateTimeField(auto_now=True, db_index=True),
        ),
    ]
