{% extends "pretixcontrol/base.html" %}
{% load i18n %}
{% load bootstrap3 %}
{% block title %}{% trans "Account settings" %}{% endblock %}
{% block content %}
<h1>{% trans "Account settings" %}</h1>
<form action="" method="post" class="form-horizontal">
    {% csrf_token %}
    {% bootstrap_form_errors form %}
    <fieldset>
        <legend>{% trans "General settings" %}</legend>
        {% bootstrap_field form.fullname layout='horizontal' %}
        {% bootstrap_field form.locale layout='horizontal' %}
        {% bootstrap_field form.timezone layout='horizontal' %}
        <div class="form-group">
            <label class="col-md-3 control-label" for="id_new_pw_repeat">{% trans "Notifications" %}</label>
            <div class="col-md-9 static-form-row">
                {% if request.user.notifications_send and request.user.notification_settings.exists %}
                    <span class="label label-success">
                        <span class="fa fa-bell-o"></span> {% trans "On" %}
                    </span>
                {% else %}
                    <span class="label label-warning">
                        <span class="fa fa-bell-slash-o"></span> {% trans "Off" %}
                    </span>
                {% endif %}
                &nbsp;
                <a href="{% url "control:user.settings.notifications" %}">
                    {% trans "Change notification settings" %}
                </a>
            </div>
        </div>
    </fieldset>
    <fieldset>
        <legend>{% trans "Login settings" %}</legend>
        {% if form.old_pw %}
            {% bootstrap_field form.old_pw layout='horizontal' %}
        {% endif %}
        {% bootstrap_field form.email layout='horizontal' %}
        {% if form.new_pw %}
            {% bootstrap_field form.new_pw layout='horizontal' %}
            {% bootstrap_field form.new_pw_repeat layout='horizontal' %}
        {% endif %}
        <div class="form-group">
            <label class="col-md-3 control-label" for="id_new_pw_repeat">{% trans "Two-factor authentication" %}</label>
            <div class="col-md-9 static-form-row">
                {% if user.require_2fa %}
                    <span class="label label-success">{% trans "Enabled" %}</span> &nbsp;
                    <a href="{% url "control:user.settings.2fa" %}">
                        {% trans "Change two-factor settings" %}
                    </a>
                {% else %}
                    <span class="label label-default">{% trans "Disabled" %}</span> &nbsp;
                    <a href="{% url "control:user.settings.2fa" %}">
                        {% trans "Enable" %}
                    </a>
                {% endif %}
            </div>
        </div>
        <div class="form-group">
            <label class="col-md-3 control-label" for="">{% trans "Authorized applications" %}</label>
            <div class="col-md-9 static-form-row">
                <a href="{% url "control:user.settings.oauth.list" %}">
                    <span class="fa fa-plug"></span>
                    {% trans "Show applications" %}
                </a>
            </div>
        </div>
        <div class="form-group">
            <label class="col-md-3 control-label" for="">{% trans "Account history" %}</label>
            <div class="col-md-9 static-form-row">
                <a href="{% url "control:user.settings.history" %}">
                    <span class="fa fa-history"></span>
                    {% trans "Show account history" %}
                </a>
            </div>
        </div>
    </fieldset>
    <div class="form-group submit-group">
        <button type="submit" class="btn btn-primary btn-save">
            {% trans "Save" %}
        </button>
    </div>
</form>
{% endblock %}
