{% load i18n %}
{% load safelink %}
{% safelink "https://pretix.eu" as pretixurl %}
{% comment %}

    Hey, you!

    Yes, you found the place in which you can remove or change the link to pretix from the shop footer.
    However, I want to politely ask you to re-consider and do not remove it.

    Creating pretix has been an huge effort that took many years and maintaining it is still a lot of
    work for us. We don't just "maintain" it in case it breaks, we are constantly adding new functionality
    every single month. However, even we need to eat something and pay our rent in some way. If we can't
    do that, there won't be anyone around supporting this project in the long term. Since you are just
    starting to rely on it do do your ticket sales -- that would be unfortunate, right?

    With pretix, we only earn money when people buy our hosting services or buy support contracts from us.
    Sure, you want to host it yourself for whatever reasons and you are free to do so. But others might see
    your ticket shop and like how it works -- and those others might not have the technical qualification
    or just the required free time to host it themselves and might turn to our services instead. Or, they
    have the technical qualification and get pointed to the open source project and turn into contributors,
    helping us to create a better ticketing software for everyone.

    Neither of that is possible, if you remove this link. If you really want, you can re-style the link,
    re-word it, put another link next to it -- but I want to politely ask you that you leave the link intact
    to enable us to create a bright future for pretix. Most likely, you will profit from that as well.

    Yours truly,
    Raphael

    P.S.: In case you are interested in a support contract or one of our additional paid features, just get
    in touch at sales@pretix.eu!

{% endcomment %}
{% with 'target="_blank" rel="noopener" href="'|add:pretixurl|add:'"'|safe as a_attr %}
    {% blocktrans trimmed %}
        <a {{ a_attr }}>event ticketing powered by pretix</a>
    {% endblocktrans %}
{% endwith %}
