# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-08-05 14:36
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ticketoutputpdf', '0004_auto_20180805_1430'),
        ('ticketoutputpdf', '0003_auto_20180805_1432'),
    ]

    operations = [
    ]
