# Generated by Django 3.0.10 on 2020-10-15 20:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0165_auto_20201015_1924'),
    ]

    operations = [
        migrations.AlterField(
            model_name='orderposition',
            name='secret',
            field=models.CharField(db_index=True, max_length=255),
        ),
    ]
