# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-03-20 10:14
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0015_auto_20160312_1924'),
    ]

    operations = [
        migrations.AddField(
            model_name='voucher',
            name='variation',
            field=models.ForeignKey(blank=True, help_text='This variation of the product select above is being used.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='vouchers', to='pretixbase.ItemVariation', verbose_name='Product variation'),
        ),
    ]
