# Generated by Django 3.0.6 on 2020-06-28 19:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0155_quota_release_after_exit'),
    ]

    operations = [
        migrations.AddField(
            model_name='cartposition',
            name='override_tax_rate',
            field=models.DecimalField(decimal_places=2, max_digits=10, null=True),
        ),
    ]
