# Generated by Django 2.2.1 on 2019-07-29 14:22

import django.db.models.deletion
from django.db import migrations, models

import pretix.base.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0130_auto_20190729_1311'),
    ]

    operations = [
        migrations.AddField(
            model_name='item',
            name='allow_waitinglist',
            field=models.BooleanField(default=True),
        ),
    ]
