{% extends "pretixpresale/event/base.html" %}
{% load i18n %}
{% load eventurl %}
{% load money %}
{% block title %}{% trans "Pay order" %}{% endblock %}
{% block content %}
    <h2>
        {% blocktrans trimmed with code=order.code %}
            Pay order: {{ code }}
        {% endblocktrans %}
    </h2>

    <form method="post" class="form-horizontal" href="">
        {% csrf_token %}

        <p>{% trans "Please confirm the following payment details." %}</p>
        <div class="row-fluid">
            <div class="panel panel-primary">
                <div class="panel-heading">
                    <div class="pull-right flip">
                        <strong>
                            {% blocktrans trimmed with total=payment.amount|money:request.event.currency %}
                                Total: {{ total }}
                            {% endblocktrans %}
                        </strong>
                    </div>
                    <h3 class="panel-title">
                        {{ payment_provider.public_name }}
                    </h3>
                </div>
                <div class="panel-body">
                    {{ payment_info }}
                </div>
            </div>
        </div>
        <div class="row checkout-button-row">
            <div class="col-md-4">
                <a class="btn btn-block btn-default btn-lg"
                   href="{% eventurl request.event "presale:event.order" secret=order.secret order=order.code %}">
                    {% trans "Cancel" %}
                </a>
            </div>
            <div class="col-md-4 col-md-offset-4">
                <button class="btn btn-block btn-primary btn-lg" type="submit">
                    {% trans "Pay now" %}
                </button>
            </div>
            <div class="clearfix"></div>
        </div>
    </form>

{% endblock %}
