# Generated by Django 2.2.4 on 2020-02-17 12:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0142_auto_20191215_1522'),
    ]

    operations = [
        migrations.AddField(
            model_name='seat',
            name='row_label',
            field=models.CharField(max_length=190, null=True),
        ),
        migrations.AddField(
            model_name='seat',
            name='seat_label',
            field=models.CharField(max_length=190, null=True),
        ),
        migrations.AlterField(
            model_name='giftcard',
            name='secret',
            field=models.CharField(db_index=True, max_length=190),
        ),
    ]
