# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-09-08 20:20
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('banktransfer', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bankimportjob',
            name='state',
            field=models.CharField(choices=[('pending', 'pending'), ('running', 'running'), ('error', 'error'), ('completed', 'completed')], default='pending', max_length=32),
        ),
        migrations.AlterField(
            model_name='banktransaction',
            name='date',
            field=models.CharField(max_length=50),
        ),
        migrations.AlterField(
            model_name='banktransaction',
            name='import_job',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='transactions', to='banktransfer.BankImportJob'),
        ),
    ]
