# Generated by Django 2.2.7 on 2019-12-15 15:22

from decimal import Decimal

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0141_seat_sorting_rank'),
    ]

    operations = [
        migrations.AddField(
            model_name='cartposition',
            name='price_before_voucher',
            field=models.DecimalField(decimal_places=2, max_digits=10, null=True),
        ),
        migrations.AddField(
            model_name='orderposition',
            name='price_before_voucher',
            field=models.DecimalField(decimal_places=2, max_digits=10, null=True),
        ),
        migrations.AddField(
            model_name='voucher',
            name='budget',
            field=models.DecimalField(decimal_places=2, max_digits=10, null=True),
        ),
    ]
