# Generated by Django 3.0.5 on 2020-04-21 07:37

import django_countries.fields
from django.db import migrations, models

import pretix.helpers.countries


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0150_auto_20200401_1123'),
    ]

    operations = [
        migrations.AddField(
            model_name='giftcard',
            name='conditions',
            field=models.TextField(null=True),
        ),
        migrations.AddField(
            model_name='giftcard',
            name='expires',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]
