{% load i18n %}
<div class="quotabox availability" data-toggle="tooltip_html" data-placement="top"
        title="{% trans "Quota:" %} {{ q.name }}<br>{% blocktrans with date=q.cached_availability_time|date:"SHORT_DATETIME_FORMAT" %}Numbers as of {{ date }}{% endblocktrans %}">
    {% if q.size|default_if_none:"NONE" == "NONE" %}
        <div class="progress">
            <div class="progress-bar progress-bar-success progress-bar-100">
            </div>
        </div>
    {% else %}
        <div class="progress">
            <div class="progress-bar progress-bar-{% if q.cached_avail.0 <= 10 or q.cached_avail.0 >= 100 %}danger{% else %}warning{% endif %} progress-bar-{{ q.inv_percent }}">
            </div>
        </div>
    {% endif %}
    <div class="numbers">
        {{ q.cached_avail.1|default_if_none:"∞" }} / {{ q.size|default_if_none:"∞" }}
    </div>
</div>
