# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-08-05 14:32
from __future__ import unicode_literals

from django.db import migrations


def cleanup(app, schema_editor):
    EventSettingsStore = app.get_model('pretixbase', 'Event_SettingsStore')
    EventSettingsStore.objects.filter(key='ticketoutput_pdf_layout').delete()


class Migration(migrations.Migration):
    dependencies = [
        ('ticketoutputpdf', '0002_auto_20180605_2022'),
    ]

    operations = [
        migrations.RunPython(cleanup, migrations.RunPython.noop)
    ]
