{% load i18n %}
<div id="ajaxerr">
</div>
<div id="loadingmodal">
    <div class="modal-card">
        <div class="modal-card-icon">
            <i class="fa fa-cog big-rotating-icon"></i>
        </div>
        <div class="modal-card-content">
            <h3></h3>
            <p class="text"></p>
            <p class="status">{% trans "If this takes longer than a few minutes, please contact us." %}</p>
        </div>
    </div>
</div>
