# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-08 10:47
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0038_auto_20160924_1448'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='require_2fa',
            field=models.BooleanField(default=False),
        ),
    ]
