{% extends "error.html" %}
{% load i18n %}
{% load static %}
{% block title %}{% trans "Permission denied" %}{% endblock %}
{% block content %}
    <i class="fa fa-fw fa-lock big-icon"></i>
    <div class="error-details">
        <h1>{% trans "Permission denied" %}</h1>
        <p>{% trans "You do not have access to this page." %}</p>
        <p>{{ exception }}</p>
        <p class="links">
            <a id='goback' href='#'>{% trans "Take a step back" %}</a>
            &middot; <a id='reload' href='#'>{% trans "Try again" %}</a>
        </p>
        {% if request.user.is_staff and not staff_session %}
            <form action="{% url 'control:user.sudo' %}?next={{ request.path|urlencode }}" method="post">
                <p>
                    {% csrf_token %}
                    <button type="submit" class="btn btn-default" id="button-sudo">
                        <i class="fa fa-id-card"></i> {% trans "Admin mode" %}
                    </button>
                </p>
            </form>
        {% endif %}
        <img src="{% static "pretixbase/img/pretix-logo.svg" %}" class="logo"/>
    </div>
{% endblock %}
