# Generated by Django 2.1 on 2018-08-07 08:41

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0098_auto_20180731_1243'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='waitinglistentry',
            options={'ordering': ('-priority', 'created'), 'verbose_name': 'Waiting list entry', 'verbose_name_plural': 'Waiting list entries'},
        ),
        migrations.AddField(
            model_name='waitinglistentry',
            name='priority',
            field=models.IntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='waitinglistentry',
            name='voucher',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='waitinglistentries', to='pretixbase.Voucher', verbose_name='Assigned voucher'),
        ),
    ]
