{% load i18n %}
<div class="panel panel-default items">
    <div class="panel-heading">
        <h3 class="panel-title">
            {% trans "Your timeline" %}
        </h3>
    </div>
    <div class="panel-body timeline">
        {% regroup timeline by date as tl_list %}
        {% for day in tl_list %}
            <div class="row {% if day.grouper < today %}text-muted{% endif %}">
                <div class="col-date">
                    <strong>{{ day.grouper|date:"SHORT_DATE_FORMAT" }}</strong>
                </div>
                <div class="col-event">
                    {% for e in day.list %}
                        <strong class="">{{ e.time|date:"TIME_FORMAT" }}</strong>
                        &nbsp;
                        <span class="{% if e.time < nearly_now %}text-muted{% endif %}">
                            {{ e.entry.description }}
                        </span>
                        {% if e.entry.edit_url %}
                            &nbsp;
                            <a href="{{ e.entry.edit_url }}" class="text-muted">
                                <span class="fa fa-edit"></span>
                            </a>
                        {% endif %}
                        {% if forloop.revcounter > 1 %}
                            <br/>
                        {% endif %}
                    {% endfor %}
                </div>
            </div>
        {% endfor %}
    </div>
</div>
