# Generated by Django 3.0.9 on 2020-10-15 16:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0163_device_security_profile'),
    ]

    operations = [
        migrations.AddField(
            model_name='subevent',
            name='last_modified',
            field=models.DateTimeField(auto_now=True, db_index=True),
        ),
    ]
