{% load i18n %}
{% load eventurl %}
{% if payment.state == "pending" %}
    <p>{% blocktrans trimmed %}
        We're waiting for an answer from the payment provider regarding your payment. Please contact us if this
        takes more than a few days.
    {% endblocktrans %}</p>
{% elif payment.state == "created" and payment_info.status == "requires_action" %}
    <p>{% blocktrans trimmed %}
        You need to confirm your payment. Please click the link below to do so or start a new payment.
    {% endblocktrans %}
        <div class="text-right flip">
            <a href="{% eventurl event "plugins:stripe:sca" order=order.code payment=payment.pk hash=payment_hash %}"
                    class="btn btn-primary">
                {% trans "Confirm payment" %}
            </a>
        </div>
        <div class="clearfix"></div>
    </p>
{% elif payment.state == "created" and payment.provider == "stripe_wechatpay" %}
    <p>{% blocktrans trimmed %}
        Please scan the barcode below to complete your WeChat payment.
        Once you have completed your payment, you can refresh this page.
    {% endblocktrans %}</p>
    <div class="text-center">
        <script type="text/plain" data-size="150" data-replace-with-qr>{{ payment_info.wechat.qr_code_url }}</script>
    </div>
{% else %}
    <p>{% blocktrans trimmed %}
        The payment transaction could not be completed for the following reason:
    {% endblocktrans %}
        <br/>
        {% if payment_info and payment_info.error %}
            {{ payment_info.message }}
        {% else %}
            {% trans "Unknown reason" %}
        {% endif %}
    </p>
{% endif %}