{% extends "pretixcontrol/event/settings_base.html" %}
{% load i18n %}
{% load bootstrap3 %}
{% block title %}{% trans "Delete tax rule" %}{% endblock %}
{% block inside %}
    <h1>{% trans "Delete tax rule" %}</h1>
    <form action="" method="post" class="form-horizontal">
        {% csrf_token %}
        {% if possible %}
            <p>{% blocktrans %}Are you sure you want to delete the tax rule <strong>{{ taxrule }}</strong>?{% endblocktrans %}</p>
        {% else %}
            <p>{% blocktrans %}You cannot delete a tax rule that is in use for a product or has been in use for any existing orders.{% endblocktrans %}</p>
        {% endif %}
        <div class="form-group submit-group">
            <a href="{% url "control:event.settings.tax" organizer=request.event.organizer.slug event=request.event.slug %}" class="btn
btn-default btn-cancel">
                {% trans "Cancel" %}
            </a>
            {% if possible %}
                <button type="submit" class="btn btn-danger btn-save">
                    {% trans "Delete" %}
                </button>
            {% endif %}
        </div>
    </form>
{% endblock %}
