# Generated by Django 3.0.11 on 2020-12-18 18:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0162_remove_seat_name'),
    ]

    operations = [
        migrations.AddField(
            model_name='cachedfile',
            name='session_key',
            field=models.TextField(null=True),
        ),
        migrations.AddField(
            model_name='cachedfile',
            name='web_download',
            field=models.BooleanField(default=True),
        ),
    ]
