# Generated by Django 2.2.9 on 2020-02-18 08:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('pretixbase', '0143_auto_20200217_1211'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoiceaddress',
            name='custom_field',
            field=models.CharField(max_length=255, null=True, blank=True),
        ),
        migrations.AddField(
            model_name='invoice',
            name='custom_field',
            field=models.CharField(max_length=255, null=True),
        ),
    ]
