# Generated by Django 3.0.9 on 2020-09-09 15:43

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0162_remove_seat_name'),
        ('banktransfer', '0006_auto_20200901_1419'),
    ]

    operations = [
        migrations.CreateModel(
            name='RefundExport',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ('datetime', models.DateTimeField(auto_now_add=True)),
                ('testmode', models.BooleanField(default=False)),
                ('rows', models.TextField(default='[]')),
                ('event', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='banktransfer_refund_exports', to='pretixbase.Event')),
                ('organizer', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='banktransfer_refund_exports', to='pretixbase.Organizer')),
                ('downloaded', models.BooleanField(default=False)),
            ],
        ),
    ]
