# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-11-26 13:00
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models

import pretix.base.validators


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0046_order_meta_info'),
    ]

    operations = [
        migrations.AddField(
            model_name='voucher',
            name='max_usages',
            field=models.PositiveIntegerField(default=1, help_text='Number of times this voucher can be redeemed.', verbose_name='Maximum usages'),
        ),
        migrations.AlterField(
            model_name='event',
            name='slug',
            field=models.CharField(max_length=50, db_index=True, help_text='Should be short, only contain lowercase letters and numbers, and must be unique among your events. This is being used in addresses and bank transfer references.', validators=[django.core.validators.RegexValidator(message='The slug may only contain letters, numbers, dots and dashes.', regex='^[a-zA-Z0-9.-]+$'), pretix.base.validators.EventSlugBanlistValidator()], verbose_name='Slug'),
        ),
        migrations.AlterField(
            model_name='organizer',
            name='slug',
            field=models.CharField(max_length=50, db_index=True, help_text='Should be short, only contain lowercase letters and numbers, and must be unique among your events. This is being used in addresses and bank transfer references.', validators=[django.core.validators.RegexValidator(message='The slug may only contain letters, numbers, dots and dashes.', regex='^[a-zA-Z0-9.-]+$'), pretix.base.validators.OrganizerSlugBanlistValidator()], verbose_name='Slug'),
        ),
        migrations.AlterField(
            model_name='voucher',
            name='redeemed',
            field=models.PositiveIntegerField(default=0, verbose_name='Redeemed'),
        ),
    ]
