# Generated by Django 2.1 on 2018-10-25 22:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0100_auto_20181023_2300'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoice',
            name='reverse_charge',
            field=models.BooleanField(default=False),
        ),
    ]
