{% load static %}
{% load compress %}
{% compress js %}
    <script type="text/javascript" src="{% static "jquery/js/jquery-2.1.1.min.js" %}"></script>
    <script type="text/javascript" src="{% static "moment/moment-with-locales.js" %}"></script>
    <script type="text/javascript" src="{% static "moment/moment-timezone-with-data-1970-2030.js" %}"></script>
    <script type="text/javascript" src="{% static "js/jquery.formset.js" %}"></script>
    <script type="text/javascript" src="{% static "bootstrap/js/bootstrap.js" %}"></script>
    <script type="text/javascript" src="{% static "datetimepicker/bootstrap-datetimepicker.js" %}"></script>
    <script type="text/javascript" src="{% static "slider/bootstrap-slider.js" %}"></script>
    <script type="text/javascript" src="{% static "pretixbase/js/details.js" %}"></script>
    <script type="text/javascript" src="{% static "pretixcontrol/js/jquery.qrcode.min.js" %}"></script>
    <script type="text/javascript" src="{% static "pretixpresale/js/widget/floatformat.js" %}"></script>
    <script type="text/javascript" src="{% static "pretixpresale/js/ui/questions.js" %}"></script>
    <script type="text/javascript" src="{% static "pretixpresale/js/ui/main.js" %}"></script>
    <script type="text/javascript" src="{% static "pretixbase/js/asynctask.js" %}"></script>
    <script type="text/javascript" src="{% static "pretixpresale/js/ui/cart.js" %}"></script>
    <script type="text/javascript" src="{% static "lightbox/js/lightbox.min.js" %}"></script>
    <script type="text/javascript" src="{% static "pretixpresale/js/ui/iframe.js" %}"></script>
{% endcompress %}
