@function tint($color, $percentage) {
  @return mix(white, $color, $percentage);
}

@function shade($color, $percentage) {
  @return mix(black, $color, $percentage);
}

$gray-darker: lighten(#000, 13.5%);
$gray-dark: lighten(#000, 20%);
$gray: lighten(#000, 33.5%);
$gray-light: lighten(#000, 60%);
$gray-lighter: lighten(#000, 93.5%);
$gray-lightest: lighten(#000, 97.25%);

$font-family-sans-serif:  "Open Sans", "OpenSans", "Helvetica Neue", Helvetica, Arial, sans-serif !default;
$text-color: #222222 !default;
$text-muted: #999999 !default;

$brand-primary: #7f5a91 !default;
$brand-success: #50a167 !default;
$brand-info: #5f9cd4 !default;
$brand-warning: #ffb419 !default;
$brand-danger: #d36060 !default;

$btn-default-border: #CCCCCC;

$border-radius-base:        3px !default;
$border-radius-large:       4px !default;
$border-radius-small:       2px !default;

$state-success-text: $brand-success;
$state-info-text: $brand-info;
$state-warning-text: $brand-warning;
$state-danger-text: $brand-danger;

$navbar-inverse-bg: #3b1c4a !default;
$navbar-inverse-link-color: white;
$navbar-inverse-link-hover-color: $gray-lighter;
$navbar-inverse-brand-hover-color: $gray-lighter;
$navbar-inverse-color: white;

$state-success-bg:               white !default;
$state-success-border:           $brand-success;
$state-info-bg:                  white !default;
$state-info-border:              $brand-info;
$state-warning-bg:               white !default;
$state-warning-border:           $brand-warning;
$state-danger-bg:                white !default;
$state-danger-border:            $brand-danger;
$panel-success-border:               tint($brand-success, 50%);
$panel-success-heading-bg:               tint($brand-success, 50%);
$panel-danger-border:               tint($brand-danger, 50%);
$panel-danger-heading-bg:               tint($brand-danger, 50%);
$panel-warning-border:               tint($brand-warning, 50%);
$panel-warning-heading-bg:               tint($brand-warning, 50%);
$panel-default-border: #e5e5e5 !default;
$panel-default-heading-bg: #e5e5e5 !default;

$body-bg: #FFFFFF !default;
$main-box-bg: #FFFFFF;
