# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-07-29 16:16
from __future__ import unicode_literals

import i18nfield.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0070_auto_20170719_0910'),
    ]

    operations = [
        migrations.AddField(
            model_name='question',
            name='help_text',
            field=i18nfield.fields.I18nTextField(blank=True, help_text='If the question needs to be explained or clarified, do it here!', null=True, verbose_name='Help text'),
        ),
        migrations.AlterField(
            model_name='invoiceaddress',
            name='vat_id',
            field=models.CharField(blank=True, help_text='Only for business customers within the EU.', max_length=255, verbose_name='VAT ID'),
        ),
    ]
