# Generated by Django 3.0.8 on 2020-07-24 07:54

from django.db import migrations, models

import pretix.base.models.base
import pretix.helpers.countries


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0157_auto_20200712_0932'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cachedfile',
            name='file',
            field=models.FileField(max_length=255, null=True, upload_to=pretix.base.models.base.cachedfile_name),
        ),
        migrations.AlterField(
            model_name='cartposition',
            name='country',
            field=pretix.helpers.countries.FastCountryField(countries=pretix.helpers.countries.CachedCountries, max_length=2, null=True),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='invoice_from_country',
            field=pretix.helpers.countries.FastCountryField(countries=pretix.helpers.countries.CachedCountries, max_length=2, null=True),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='invoice_to_country',
            field=pretix.helpers.countries.FastCountryField(countries=pretix.helpers.countries.CachedCountries, max_length=2, null=True),
        ),
        migrations.AlterField(
            model_name='invoiceaddress',
            name='country',
            field=pretix.helpers.countries.FastCountryField(countries=pretix.helpers.countries.CachedCountries, max_length=2),
        ),
        migrations.AlterField(
            model_name='orderposition',
            name='country',
            field=pretix.helpers.countries.FastCountryField(countries=pretix.helpers.countries.CachedCountries, max_length=2, null=True),
        ),
        migrations.AlterField(
            model_name='taxrule',
            name='home_country',
            field=pretix.helpers.countries.FastCountryField(countries=pretix.helpers.countries.CachedCountries, max_length=2),
        ),
    ]
