{% load i18n %}

<p>
    {% url "control:event.order" organizer=event.organizer.slug event=event.slug code=data.order as ourl %}
    {% blocktrans trimmed with payment=data.resource.id order="<a href='"|add:ourl|add:"'>"|add:data.order|add:"</a>"|safe %}
        PayPal reported that the payment {{ payment }} has been refunded or reversed.
        Do you want to mark the matching order ({{ order }}) as refunded?
    {% endblocktrans %}
</p>
