# -*- coding: utf-8 -*-
# Generated by Django 1.9.8 on 2016-08-15 12:54
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0026_order_comment'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='invoiceaddress',
            name='phone',
        ),
        migrations.AlterField(
            model_name='invoiceaddress',
            name='name',
            field=models.CharField(blank=True, max_length=255, verbose_name='Full name'),
        ),
        migrations.AlterField(
            model_name='item',
            name='hide_without_voucher',
            field=models.BooleanField(default=False, help_text='This product will be hidden from the event page until the user enters a voucher code that is specifically tied to this product (and not via a quota).', verbose_name='This product will only be shown if a voucher matching the product is redeemed.'),
        ),
    ]
