# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-04-23 09:44
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0020_auto_20160421_1943'),
        ('pretixbase', '0021_auto_20160418_2117'),
    ]

    operations = [
    ]
