# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-08-16 12:42
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0027_auto_20160815_1254'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='cartposition',
            name='base_price',
        ),
        migrations.RemoveField(
            model_name='cartposition',
            name='voucher_discount',
        ),
        migrations.RemoveField(
            model_name='orderposition',
            name='base_price',
        ),
        migrations.RemoveField(
            model_name='orderposition',
            name='voucher_discount',
        ),
    ]
