{% extends "pretixpresale/organizers/base.html" %}
{% load i18n %}
{% load rich_text %}
{% load eventurl %}
{% load urlreplace %}
{% block title %}{% trans "Event overview" %}{% endblock %}
{% block content %}
    {% if organizer_homepage_text %}
        <div>
            {{ organizer_homepage_text | rich_text }}
        </div>
    {% endif %}
    <h3>{{ date|date:"F Y" }}</h3>
    <form class="form-inline" method="get" id="monthselform" action="{% eventurl request.organizer "presale:organizer.index" %}">
        {% for f, v in request.GET.items %}
            {% if f != "month" and f != "year" %}
                <input type="hidden" name="{{ f }}" value="{{ v }}">
            {% endif %}
        {% endfor %}
        <div class="row">
            <div class="col-sm-4 col-xs-12 text-left flip">
                <div class="btn-group" role="group">
                    <a href="?{% url_replace request "style" "list" %}" type="button" class="btn btn-default">
                        <span class="fa fa-list"></span>
                        {% trans "List" %}
                    </a>
                    <a href="?{% url_replace request "style" "week" %}" type="button"
                            class="btn btn-default">
                        <span class="fa fa-calendar"></span>
                        {% trans "Week" %}
                    </a>
                    <a href="?{% url_replace request "style" "calendar" %}"
                            type="button"
                            class="btn btn-default active">
                        <span class="fa fa-calendar"></span>
                        {% trans "Month" %}
                    </a>
                </div>
                <a href="{% eventurl request.organizer "presale:organizer.ical" %}?{% url_replace request "locale" request.LANGUAGE_CODE "style" "" "month" "" "year" "" %}"
                        class="btn btn-default">
                    <span class="fa fa-calendar-plus-o"></span>
                    {% trans "iCal" %}
                </a>
            </div>
            <div class="col-sm-4 col-xs-12 text-center">
                <select name="month" class="form-control">
                    {% for m in months %}
                        <option value="{{ m|date:"m" }}" {% if m == date %}selected{% endif %}>{{ m|date:"F" }}</option>
                    {% endfor %}
                </select>
                <select name="year" class="form-control">
                    {% for y in years %}
                        <option value="{{ y }}" {% if y == date.year %}selected{% endif %}>{{ y }}</option>
                    {% endfor %}
                </select>
                <button type="submit" class="js-hidden btn btn-default">
                    {% trans "Go" %}
                </button>
            </div>
            <div class="col-sm-4 hidden-xs text-right flip">
                <a href="?{% url_replace request "year" before.year "month" before.month %}" class="btn btn-default">
                    <span class="fa fa-arrow-left"></span>
                    {{ before|date:"F Y" }}
                </a>
                <a href="?{% url_replace request "year" after.year "month" after.month %}" class="btn btn-default">
                    <span class="fa fa-arrow-right"></span>
                    {{ after|date:"F Y" }}
                </a>
            </div>
        </div>
    </form>
    {% include "pretixpresale/fragment_calendar.html" with show_avail=request.organizer.settings.event_list_availability %}

    {% if multiple_timezones %}
        <div class="alert alert-info">
            {% blocktrans trimmed %}
                Note that the events in this view are in different timezones.
            {% endblocktrans %}
        </div>
    {% endif %}
{% endblock %}
