<div class="form-inline helper-display-inline slug-widget">
    {{ pre }}
    <input type="{{ widget.type }}" name="{{ widget.name }}"
            {% if widget.value != None %}value="{{ widget.value }}"{% endif %}
            {% include "django/forms/widgets/attrs.html" %}
    />
    /
</div>
