.sidebar-nav li > a > .fa {
  color: $navbar-inverse-bg;
}
.sidebar-nav li > a > svg {
  position: relative;
  top: 3px;
  path {
    fill: $navbar-inverse-bg;
  }
}
.nav .testmode a {
  background: $brand-warning;
  color: black;
  font-weight: bold;
  &:hover, &:active, &:focus {
    background: $btn-warning-border;
  }
}

.btn {
  border-width: 0px;
}

.btn-default {
  box-shadow: 0px 0px 0px 1px #cccccc inset;
  box-sizing: border-box;
}

.btn-default:hover, .btn-default:focus {
  box-shadow: 0px 0px 0px 1px #cccccc inset, inset 0 1px 3px 0 #BFBFBF;
  background: $btn-default-bg;
}

.btn-default:active, .btn-default:active:hover, .btn-default:active:focus {
  box-shadow: 0px 0px 0px 1px #cccccc inset, inset 0 1px 8px 0 #BFBFBF;
  background: $btn-default-bg;
  outline: 0;
}

.btn-primary:hover, .btn-primary:focus {
  box-shadow: inset 0 1px 3px 0 shade($brand-primary, 25%);
  background: $btn-primary-bg;
}

.btn-primary:active, .btn-primary:active:hover, .btn-primary:active:focus {
  box-shadow: inset 0 1px 8px 0 shade($brand-primary, 25%);
  background: $btn-primary-bg;
  outline: 0;
}

.btn-success:hover, .btn-success:focus {
  box-shadow: inset 0 1px 3px 0 shade($brand-success, 25%);
  background: $btn-success-bg;
}

.btn-success:active, .btn-success:active:hover, .btn-success:active:focus {
  box-shadow: inset 0 1px 8px 0 shade($brand-success, 25%);
  background: $btn-success-bg;
  outline: 0;
}

.btn-warning:hover, .btn-warning:focus {
  box-shadow: inset 0 1px 3px 0 shade($brand-warning, 25%);
  background: $btn-warning-bg;
}

.btn-warning:active, .btn-warning:active:hover, .btn-warning:active:focus {
  box-shadow: inset 0 1px 8px 0 shade($brand-warning, 25%);
  background: $btn-warning-bg;
  outline: 0;
}

.btn-danger:hover, .btn-danger:focus {
  box-shadow: inset 0 1px 8px 0 shade($brand-danger, 25%);
  background: $btn-danger-bg;
}

.btn-danger:active, .btn-danger:active:hover, .btn-danger:active:focus {
  box-shadow: inset 0 1px 3px 0 shade($brand-danger, 25%);
  background: $btn-danger-bg;
  outline: 0;
}

.btn-invisible {
  background: transparent;
  border: transparent;
}

.alert-primary, .alert-warning, .alert-info, .alert-success, .alert-danger {
  color: #3b3b3b;
}
.panel-danger > .panel-heading, .panel-success > .panel-heading, .panel-default > .panel-heading, .panel-info > .panel-heading, .panel-warning > .panel-heading {
  color: #000000;
}
.panel-primary .panel-heading {
  color: white;
}


.payments dt {
  text-align: left;
  font-weight: normal;
}


.alert {
  text-align: left;
}
.alert-legal {
  border-color: $state-info-border;
}
.alert-success, .alert-danger, .alert-info, .alert-warning, .alert-legal {
  position: relative;
  padding-left: 65px;

  &::before {
    color: white;
    position: absolute;
    display: block;
    left: 0;
    top: 0;
    width: 50px;
    padding-top: 13px;
    text-align: center;
    height: 100%;
    font: normal normal normal 14px FontAwesome;
    font-size: 24px;
    text-rendering: auto; // optimizelegibility throws things off #1094
    -webkit-font-smoothing: antialiased;
    -moz-osx-font-smoothing: grayscale;
  }
}
.sr-only.alert::before {
  background: none !important;
}
.alert-success::before {
  background: $state-success-border;
  content: "\f00c";
}
.alert-info::before {
  background: $state-info-border;
  content: "\f129";
}
.alert-warning::before {
  background: $state-warning-border;
  content: "\f071";
  font-size: 22px;
  padding-top: 14px;
}
.alert-legal::before {
  background: $state-info-border;
  content: "\f0e3";
}
.alert-danger::before {
  background: $state-danger-border;
  content: "\f071";
  font-size: 22px;
  padding-top: 14px;
}
.alert-primary::before {
  background: $brand-primary !important;
}
.alert-primary {
  border-color: $brand-primary !important;
}

.progress-bar {
  box-shadow: none;
  -webkit-box-shadow: none;
}

.pagination > li.page-current > a:hover, .pagination > li.page-current > a:focus {
  color: #7f5a91;
  background-color: #fff;
  border: 1px solid #ddd;
}

svg.svg-icon {
  position: relative;
  top: 2px;
}

@include table-row-variant('success', lighten($brand-success, 40%));
@include table-row-variant('info', lighten($brand-info, 30%));
@include table-row-variant('warning', lighten($brand-warning, 40%));
@include table-row-variant('danger', lighten($brand-danger, 30%));
