# Generated by Django 2.2.9 on 2020-03-21 15:12

from django.db import migrations, models

import pretix.base.models.auth


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0146_giftcardtransaction_text'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='session_token',
            field=models.CharField(default=pretix.base.models.auth.generate_session_token, max_length=32),
        ),
    ]
