# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-01-15 14:26
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0079_auto_20180115_0855'),
    ]

    operations = [
        migrations.AddField(
            model_name='question',
            name='ask_during_checkin',
            field=models.BooleanField(default=False, help_text='Supported by pretixdroid 1.8 and newer or pretixdesk 0.2 and newer.', verbose_name='Ask during check-in instead of during registration'),
        ),
    ]
