{% extends "pretixcontrol/base.html" %}
{% load i18n %}
{% load bootstrap3 %}
{% load hierarkey_form %}
{% load formset_tags %}
{% block title %}{% trans "Stripe Connect" %}{% endblock %}
{% block content %}
    <h1>
        {% trans "Stripe Connect" %}
    </h1>

    <form action="" method="post" class="form-horizontal" enctype="multipart/form-data">
        {% csrf_token %}
        {% url "control:global.settings" as g_url %}
        {% propagated request.organizer g_url "payment_stripe_connect_app_fee_percent" "payment_stripe_connect_app_fee_min" "payment_stripe_connect_app_fee_max" %}
            {% bootstrap_form form layout="control" %}
        {% endpropagated %}
        <div class="form-group submit-group">
            <button type="submit" class="btn btn-primary btn-save">
                {% trans "Save" %}
            </button>
        </div>
    </form>
{% endblock %}
