# Generated by Django 3.0.10 on 2020-10-24 15:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0168_auto_20201023_1447'),
    ]

    operations = [
        migrations.AddField(
            model_name='checkinlist',
            name='gates',
            field=models.ManyToManyField(to='pretixbase.Gate'),
        ),
    ]
