# Generated by Django 2.2.1 on 2019-05-30 11:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0123_auto_20190530_1035'),
    ]

    operations = [
        migrations.AddField(
            model_name='seat',
            name='seat_guid',
            field=models.CharField(db_index=True, default=None, max_length=190),
            preserve_default=False,
        ),
    ]
