{% extends "pretixcontrol/organizers/base.html" %}
{% load i18n %}
{% load bootstrap3 %}
{% block inner %}
    <h1>
        {% blocktrans trimmed with card=card.secret %}
            Gift card: {{ card }}
        {% endblocktrans %}
        {% if card.testmode %}
            <span class="label label-warning">{% trans "TEST MODE" %}</span>
        {% endif %}
    </h1>
    <form class="form-horizontal" action="" method="post">
        {% csrf_token %}
        {% bootstrap_form_errors form %}
        {% bootstrap_field form.expires layout="control" %}
        {% bootstrap_field form.conditions layout="control" %}
        <div class="form-group submit-group">
            <button type="submit" class="btn btn-primary btn-save">
                {% trans "Save" %}
            </button>
        </div>
    </form>
{% endblock %}
