# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2017-01-13 14:07
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models

import pretix.base.models.orders


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0058_auto_20170107_1533'),
    ]

    operations = [
        migrations.CreateModel(
            name='CachedCombinedTicket',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('provider', models.CharField(max_length=255)),
                ('type', models.CharField(max_length=255)),
                ('extension', models.CharField(max_length=255)),
                ('file', models.FileField(blank=True, null=True, upload_to=pretix.base.models.orders.cachedcombinedticket_name)),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='pretixbase.Order')),
            ],
        ),
    ]
