# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-11-08 15:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0044_auto_20161101_1610'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cartposition',
            name='expires',
            field=models.DateTimeField(db_index=True, verbose_name='Expiration date'),
        ),
        migrations.AlterField(
            model_name='voucher',
            name='redeemed',
            field=models.BooleanField(db_index=True, default=False, verbose_name='Redeemed'),
        ),
        migrations.AlterField(
            model_name='voucher',
            name='valid_until',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Valid until'),
        ),
    ]
