syntax = "proto3";

option java_package = "eu.pretix.libpretixsync.crypto.sig1";
option java_outer_classname = "TicketProtos";

message Ticket {
    string seed = 1;
    int64 item = 2;
    int64 variation = 3;
    int64 subevent = 4;
}
