{% extends "pretixcontrol/base.html" %}
{% load i18n %}
{% load bootstrap3 %}
{% block title %}{% trans "Disable application" %}{% endblock %}
{% block content %}
    <h1>{% trans "Disable application" %}</h1>
        <form action="" method="post" class="form-horizontal">
            {% csrf_token %}
            <p>{% blocktrans %}Are you sure you want to disable the application <strong>{{ application }}</strong> permanently?{% endblocktrans %}</p>
            <div class="form-group submit-group">
                <a href="{% url "control:user.settings.oauth.apps" %}" class="btn btn-default btn-cancel">
                    {% trans "Cancel" %}
                </a>
                <button type="submit" class="btn btn-danger btn-save">
                    {% trans "Disable" %}
                </button>
            </div>
        </form>
{% endblock %}
