# Generated by Django 3.0.9 on 2020-08-24 07:29

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0161_order_changes_retain_old_default'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='seat',
            name='name',
        ),
    ]
