# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-06-04 11:19
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('pretixbase', '0093_auto_20180528_1432'),
        ('pretixapi', '0001_initial')
    ]

    operations = [
    ]
