html.rtl {
  #loadingmodal .modal-card .modal-card-icon, #ajaxerr .modal-card .modal-card-icon {
    float: right;
  }
  #loadingmodal .modal-card .modal-card-content, #ajaxerr .modal-card .modal-card-content {
    margin-right: 160px;
    margin-left: 0;
    text-align: right;
  }
  .alert-success, .alert-danger, .alert-info, .alert-warning, .alert-legal {
    padding-right: 65px;
    padding-left: 15px;
  }
  .alert {
    text-align: right;
  }
  .alert-success::before, .alert-danger::before, .alert-info::before, .alert-warning::before, .alert-legal::before {
    left: inherit;
    right: 0;
  }
}

@media(min-width: $screen-sm-min) {
  html.rtl .nameparts-form-group {
    input, select {
      border-radius: $border-radius-base;
    }
    input:not(:first-child), select:not(:first-child) {
      border-bottom-right-radius: 0;
      border-top-right-radius: 0;
    }
    input:not(:last-child), select:not(:last-child)  {
      border-bottom-left-radius: 0;
      border-top-left-radius: 0;
    }
  }
}
@media (max-width: 700px) {
  html.rtl {
    #loadingmodal .modal-card .modal-card-content, #ajaxerr .modal-card .modal-card-content {
      margin-right: 0;
    }
  }
}
