# Generated by Django 2.1 on 2018-08-12 14:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('stripe', '0002_referencedstripeobject_payment'),
    ]

    operations = [
        migrations.CreateModel(
            name='RegisteredApplePayDomain',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('domain', models.CharField(max_length=190)),
                ('account', models.CharField(max_length=190)),
            ],
        ),
    ]
