# Generated by Django 2.2.4 on 2019-08-30 15:13

import django.db.models.deletion
from django.db import migrations, models

import pretix.base.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('pretixbase', '0132_auto_20190808_1253'),
    ]

    operations = [
        migrations.AddField(
            model_name='question',
            name='print_on_invoice',
            field=models.BooleanField(default=False),
        ),
    ]
